local function printerr(message)
	print("\169255000000DIVERSITY: " .. message)
end

if ((_CS2DVERSION or "1.0.0.1") < "1.0.0.2") then
	return printerr("attempt to use Diversity with invalid version (1.0.0.2 or later expected" .. (_CS2DVERSION and ", got " .. _CS2DVERSION or "") .. ")")
end

local config = {}

do
	local file, err = io.open((...) .. "/configurations.cfg")
	
	if (file) then
		local configs = {"loaddirectory", "officiallanguages"}
		local configsexecuted = {}
		
		for _, name in ipairs(configs) do
			configsexecuted[name] = false
		end
		
		for line in file:lines() do
			local name, value = line:match("#(%w+) (.+)")
			
			if (name) then
				if (name == "loaddirectory") then
					if (io.isdir(value)) then
						config[name] = value
					else
						return printerr("configurations.cfg: #" .. name .. ": no '" .. value .. "' directory")
					end
				elseif (name == "officiallanguages") then
					local languages = {}
					
					for value in value:gmatch("[^%s]+") do
						languages[#languages + 1] = value
					end
					
					config[name] = languages
				end
				
				configsexecuted[name] = true
			end
		end
		
		
		file:close()
		
		local success = true
		local configsnotexecuted = {}
		
		for _, name in ipairs(configs) do
			if (not configsexecuted[name]) then
				if (success) then
					success = false
				end
				
				configsnotexecuted[#configsnotexecuted + 1] = "#" .. name
			end
		end
		
		if (not success) then
			local plural = #configsnotexecuted > 1
			
			return printerr("configurations.cfg: " .. table.concat(configsnotexecuted, ", ") .. " " .. (plural and "are" or "is") .. " either not exist or ha" .. (plural and "ve" or "s") .. " no value")
		end
	else
		return printerr(err)
	end
end

local language = {}

for item in io.enumdir(config.loaddirectory) do
	local name, extension = item:match("([^/]+)%.([^%.]+)$")
	
	if (name and extension == "txt") then
		local file, err = io.open(config.loaddirectory .. "/" .. name .. ".txt")
		
		if (not file) then
			return printerr(config.loaddirectory .. "/" .. name .. ".txt: " .. err)
		end
		
		local currentlanguage = {}
		local currentline = 1
		
		for line in file:lines() do
			currentlanguage[currentline] = line
			currentline = currentline + 1
		end
		
		file:close()
		
		language[name] = currentlanguage
	end
end

do
	local success = true
	local officiallanguages = {}
	
	for _, officiallanguage in ipairs(config.officiallanguages) do
		if (not language[officiallanguage]) then
			if (success) then
				success = false
			end
			
			officiallanguages[#officiallanguages + 1] = officiallanguage
		end
	end
	
	if (not success) then
		local plural = #officiallanguages > 1
		
		return printerr("official " .. table.concat(officiallanguages, ", ") .. " language" .. (plural and "s" or "") .. " " .. (plural and "are" or "is") .. " not exist")
	end
end

return function(name, line, ...)
	local str = language[name] and language[name][line] or nil
	
	if (str) then
		for index, arg in ipairs({...}) do
			str = str:gsub("$" .. index, arg)
		end
	end
	
	return str
end